<?php
/**
 * Copyright 2011-2023 Twinkle Toes Software, LLC
 */

require_once(ROOT_DIR . 'Pages/Admin/AdminPage.php');
require_once(ROOT_DIR . 'Pages/IPageable.php');
require_once(ROOT_DIR . 'Pages/Ajax/AutoCompletePage.php');
require_once(ROOT_DIR . 'Presenters/Admin/ManageSchedulesPresenter.php');
require_once(ROOT_DIR . 'Domain/Access/ScheduleRepository.php');
require_once(ROOT_DIR . 'lib/Application/Attributes/namespace.php');


interface IManageResourcesPage extends IActionPage
{
    /**
     * @return int
     */
    public function GetResourceId();

    /**
     * @return int
     */
    public function GetApiResourceId();

    /**
     * @return string[]
     */
    public function GetApiImageNames();

    /**
     * @return UploadedFile[]
     */
    public function GetApiUploadedImages();

    /**
     * @param string $qrCodeImageUrl
     * @param string $resourceName
     */
    public function ShowQRCode($qrCodeImageUrl, $resourceName);

}

class ManageResourcesPage extends ActionPage implements IManageResourcesPage
{
    /**
     * @var ManageResourcesPresenter
     */
    protected $presenter;

    public function __construct()
    {
        parent::__construct('ManageResources', 2);
        $this->presenter = new ManageResourcesPresenter(
            $this,
            new ResourceRepository(),
            new ScheduleRepository(),
            new GroupRepository(),
            new AttributeService(new AttributeRepository()),
            new ReservationViewRepository(),
            new UserRepository(),
        );

        $this->Set('CreditsEnabled', Configuration::Instance()->GetSectionKey(ConfigSection::CREDITS, ConfigKeys::CREDITS_ENABLED, new BooleanConverter()));
        $page = basename($_SERVER['REQUEST_URI']);
        $this->Set('Endpoint', empty($page) || !BookedStringHelper::Contains($page, ".php") ? "manage_resources.php" : $page);

        $url = $this->server->GetUrl();
        $exportUrl = BookedStringHelper::Contains($url, '?') ? $url . '&dr=export' : $this->server->GetRequestUri() . '?dr=export';
        $this->Set('ExportUrl', $exportUrl);
    }

    public function ProcessPageLoad()
    {
        $this->Display('Admin/Resources/manage-resources-spa.tpl');
    }

    public function ProcessApiCall($json)
    {
        $this->presenter->ProcessApi($json);
    }

    public function ProcessDataRequest($dataRequest)
    {
    }

    public function ProcessAction()
    {
        $this->presenter->ProcessAction();
    }

    public function GetResourceId()
    {
        $id = $this->GetQuerystring(QueryStringKeys::RESOURCE_ID);
        if (empty($id)) {
            $id = $this->GetForm(FormKeys::PK);
        }

        return $id;
    }

    public function ShowQRCode($qrCodeImageUrl, $resourceName)
    {
        $this->Set('QRImageUrl', $qrCodeImageUrl);
        $this->Set('ResourceName', $resourceName);
        $this->Set('HelpText', "Scan this code to quickly book this resource or check into an ongoing reservation.");

        $this->Display('Admin/Resources/show_resource_qr.tpl');
    }

    public function GetApiResourceId()
    {
        return $this->GetForm("id");
    }

    public function GetApiImageNames()
    {
        return $this->GetForm("images");
    }

    public function GetApiUploadedImages()
    {
        return $this->server->GetFiles("files");
    }
}

class ResourceFilterValues
{
    public $ResourceNameFilter;
    public $ScheduleIdFilter;
    public $ResourceTypeFilter;
    public $ResourceStatusFilterId;
    public $ResourceStatusReasonFilterId;
    public $CapacityFilter;
    public $RequiresApprovalFilter;
    public $AutoPermissionFilter;
    public $AllowMultiDayFilter;
    public $Attributes = array();

    /**
     * @param AttributeFormElement[] $attributeFormElements
     */
    public function SetAttributes($attributeFormElements)
    {
        foreach ($attributeFormElements as $e) {
            $this->SetAttributeValue($e->Id, $e->Value);
        }
    }

    public function SetAttributeValue($id, $value)
    {
        $this->Attributes[$id] = $value;
    }

    public function GetAttributeValue($id)
    {
        if (array_key_exists($id, $this->Attributes)) {
            return $this->Attributes[$id];
        }

        return null;
    }

    /**
     * @param CustomAttribute[] $customAttributes
     * @return ISqlFilter
     */
    public function AsFilter($customAttributes)
    {
        $filter = new SqlFilterNull();
        if (!empty($this->ResourceNameFilter)) {
            $filter->_And(new SqlFilterLike(new SqlFilterColumn(TableNames::RESOURCES_ALIAS,
                ColumnNames::RESOURCE_NAME),
                $this->ResourceNameFilter));
        }
        if (!empty($this->ScheduleIdFilter)) {
            $filter->_And(new SqlFilterEquals(new SqlFilterColumn(TableNames::RESOURCES_ALIAS,
                ColumnNames::SCHEDULE_ID), $this->ScheduleIdFilter));
        }
        if (!empty($this->ResourceTypeFilter)) {
            $filter->_And(new SqlFilterEquals(new SqlFilterColumn(TableNames::RESOURCES_ALIAS,
                ColumnNames::RESOURCE_TYPE_ID),
                $this->ResourceTypeFilter));
        }
        if ($this->ResourceStatusFilterId != '') {
            $filter->_And(new SqlFilterEquals(new SqlFilterColumn(TableNames::RESOURCES_ALIAS,
                ColumnNames::RESOURCE_STATUS_ID),
                $this->ResourceStatusFilterId));
        }
        if ($this->ResourceStatusReasonFilterId != '') {
            $filter->_And(new SqlFilterEquals(new SqlFilterColumn(TableNames::RESOURCES_ALIAS,
                ColumnNames::RESOURCE_STATUS_REASON_ID),
                $this->ResourceStatusReasonFilterId));
        }
        if (!empty($this->CapacityFilter)) {
            $filter->_And(new SqlFilterGreaterThan(new SqlFilterColumn(TableNames::RESOURCES_ALIAS,
                ColumnNames::RESOURCE_MAX_PARTICIPANTS),
                $this->CapacityFilter, true));
        }
        if ($this->RequiresApprovalFilter != '') {
            $filter->_And(new SqlFilterEquals(new SqlFilterColumn(TableNames::RESOURCES_ALIAS,
                ColumnNames::RESOURCE_REQUIRES_APPROVAL),
                $this->RequiresApprovalFilter));
        }
        if ($this->AutoPermissionFilter != '') {
            $filter->_And(new SqlFilterEquals(new SqlFilterColumn(TableNames::RESOURCES_ALIAS,
                ColumnNames::RESOURCE_AUTOASSIGN),
                $this->AutoPermissionFilter));
        }
        if ($this->AllowMultiDayFilter != '') {
            $filter->_And(new SqlFilterEquals(new SqlFilterColumn(TableNames::RESOURCES_ALIAS,
                ColumnNames::RESOURCE_ALLOW_MULTIDAY),
                $this->AllowMultiDayFilter));
        }

        if (!empty($this->Attributes)) {
            $filteringAttributes = false;
            $attributeDefinitions = array();
            foreach ($customAttributes as $a) {
                $attributeDefinitions[$a->Id()] = $a;
            }

            $f = new SqlFilterFreeForm('`' . ColumnNames::RESOURCE_ID . '` IN (SELECT `a0`.`' . ColumnNames::ATTRIBUTE_ENTITY_ID . '` FROM `' . TableNames::CUSTOM_ATTRIBUTE_VALUES . '` `a0` ');

            $attributeFragment = new SqlFilterNull();

            /** @var $attribute Attribute */
            foreach ($this->Attributes as $id => $value) {
                if ($value == null || $value == '' || !array_key_exists($id, $attributeDefinitions)) {
                    continue;
                }
                $filteringAttributes = true;
                $attribute = $attributeDefinitions[$id];
                $attributeId = new SqlRepeatingFilterColumn("a$id", ColumnNames::CUSTOM_ATTRIBUTE_ID, $id);
                $attributeValue = new SqlRepeatingFilterColumn("a$id", ColumnNames::CUSTOM_ATTRIBUTE_VALUE, $id);

                $idEquals = new SqlFilterEquals($attributeId, $id);
                $f->AppendSql('LEFT JOIN `' . TableNames::CUSTOM_ATTRIBUTE_VALUES . '` `a' . $id . '` ON `a0`.`entity_id` = `a' . $id . '`.`entity_id` ');
                if ($attribute->Type() == CustomAttributeTypes::MULTI_LINE_TEXTBOX || $attribute->Type() == CustomAttributeTypes::SINGLE_LINE_TEXTBOX) {
                    $attributeFragment->_And($idEquals->_And(new SqlFilterLike($attributeValue, $value)));
                } else {
                    $attributeFragment->_And($idEquals->_And(new SqlFilterEquals($attributeValue, $value)));
                }
            }

            $f->AppendSql("WHERE [attribute_list_token] )");
            $f->Substitute('attribute_list_token', $attributeFragment);

            if ($filteringAttributes) {
                $filter->_And($f);
            }
        }

        return $filter;
    }
}